import processing.serial.*;
import themidibus.*;

Serial serialPort;
MidiBus midiPort;

/* !!! Configuration start !!! */

/* Initial RGB LED status */
char ledStatus[] = {1, 1, 1};

/* Initial mode */
char curMode = 0;

/* Maximum frequency */
char maxFreq = 20;

/* Automatic hue reset */
/* Resets hue when changing mode */
/* It's better to manually reset it anyway */
/* 0 -> no reset, 1 -> reset */
char autoHueReset = 0;

/* Serial port name */
String serialName = "COM7";

/* MIDI In device name */
String midiInName = "Hercules DJ Control Steel";

/* MIDI Out device name */
String midiOutName = "Hercules DJ Control Steel";

/* !!! Configuration end !!! */

char firstLoop = 1;

void setup() {
  size(400, 400);
  background(255, 0, 0); /* Red */
  
  println(Serial.list());
  MidiBus.list();
  
  serialPort = new Serial(this, serialName, 57600); /* This will reset the Arduino */
}

void waitInit() {
  char chr;
  char i = 0;
  String init = "Ready!";
  
  while(true) {
    if(serialPort.available() > 0) {
      chr = (char) serialPort.read();
      if(chr == init.charAt(i)) {
        i++;
        if(i == init.length()) {
          serialPort.clear(); /* Cleanup serial port */
          
          /* Sync LED status */
          serialPort.write(0x04 + (ledStatus[0] << 3)); /* 0000S100 */
          serialPort.write(0x02 + (ledStatus[1] << 3)); /* 0000S010 */
          serialPort.write(0x01 + (ledStatus[2] << 3)); /* 0000S001 */
          
          /* Sync current mode */
          serialPort.write(0x10 + curMode); /* 0001MMMM */
          
          /* Set maximum frequency */
          serialPort.write(0x80); /* 10000000 */
          serialPort.write(maxFreq);
          
          /* Reset hue */
          serialPort.write(0x20); /* 00100000 */
          
          midiPort = new MidiBus(this, midiInName, midiOutName);
          
          /* Set console LEDs */
          midiPort.sendControllerChange(0, 0x0E, (ledStatus[0] + 1 ) % 2 * 0x7F);
          midiPort.sendControllerChange(0, 0x0F, (ledStatus[1] + 1 ) % 2 * 0x7F);
          midiPort.sendControllerChange(0, 0x10, (ledStatus[2] + 1 ) % 2 * 0x7F);
          midiPort.sendControllerChange(0, 0x01, 0);
          midiPort.sendControllerChange(0, 0x02, 0);
          midiPort.sendControllerChange(0, 0x04, 0);
          
          /* Update all analogs */
          midiPort.sendControllerChange(0, 0x70, 0x7F);
          
          background(0, 255, 0); /* Green */
          
          return;
        }
      }
      else {
        i = 0;
      }
    }
    
    delay(100);
  }
}

String myString;

void draw() {
  while(serialPort.available() > 0) {
    myString = serialPort.readString();
    if(myString != null) {
      println(myString);
    }
  }
  
  if(firstLoop == 1) {
    waitInit();
    firstLoop = 0;
  }
}

void noteOn(int channel, int pitch, int velocity) {
  return;
}

void noteOff(int channel, int pitch, int velocity) {
  return;
}

void controllerChange(int channel, int number, int value) {
  /* No need to check for channel */
  switch(number) {
    case 0x32: /* Left volume fader - set strobe frequency */
      serialPort.write(0x90); /* 10010000 */
      serialPort.write((2 * value) + (value / 127));
      break;
    case 0x38: /* Main volume fader - set brightness */
      serialPort.write(0xC0); /* 11000000 */
      serialPort.write((2 * value) + (value / 127));
      break;
    case 0x3C: /* Right volume fader - set fading speed */
      serialPort.write(0x91); /* 10010001 */
      serialPort.write(255 - ((2 * value) + (value / 127)));
      break;
    case 0x31: /* Left pitch fader - set strobe duty cycle */
      serialPort.write(0xA0); /* 10100000 */
      serialPort.write((2 * value) + (value / 127));
      break;
    case 0x34: /* Left high knob - set red PWM */
      serialPort.write(0xB4); /* 10110100 */
      serialPort.write((2 * value) + (value / 127));
      break;
    case 0x35: /* Left mid knob - set green PWM */
      serialPort.write(0xB2); /* 10110010 */
      serialPort.write((2 * value) + (value / 127));
      break;
    case 0x36: /* Left bass knob - set blue PWM */
      serialPort.write(0xB1); /* 10110001 */
      serialPort.write((2 * value) + (value / 127));
      break;
    case 0x0E: /* Left high kill - on/off red */
      if(value == 0x7F) {
        ledStatus[0] = (char) ((ledStatus[0] + 1) % 2);
        serialPort.write(0x04 + (ledStatus[0] << 3)); /* 0000S100 */
        midiPort.sendControllerChange(0, 0x0E, ((ledStatus[0] + 1) % 2) * 0x7F); /* Control console LED */
      }
      break;
    case 0x0F: /* Left mid kill - on/off green */
      if(value == 0x7F) {
        ledStatus[1] = (char) ((ledStatus[1] + 1) % 2);
        serialPort.write(0x02 + (ledStatus[1] << 3)); /* 0000S010 */
        midiPort.sendControllerChange(0, 0x0F, ((ledStatus[1] + 1) % 2) * 0x7F); /* Control console LED */
      }
      break;
    case 0x10: /* Left bass kill - on/off blue */
      if(value == 0x7F) {
        ledStatus[2] = (char) ((ledStatus[2] + 1) % 2);
        serialPort.write(0x01 + (ledStatus[2] << 3)); /* 0000S001 */
        midiPort.sendControllerChange(0, 0x10, ((ledStatus[2] + 1) % 2) * 0x7F); /* Control console LED */
      }
      break;
    case 0x01: /* Left pad 1 - fading mode */
      if(value == 0x7F) {
        if(curMode == 1) {
          curMode = 0;
          midiPort.sendControllerChange(0, 0x01, 0);
        }
        else {
          curMode = 1;
          midiPort.sendControllerChange(0, 0x01, 0x7F);
          midiPort.sendControllerChange(0, 0x04, 0);
        }
        
        if(autoHueReset == 1)
          serialPort.write(0x20); /* 00100000 */
        
        serialPort.write(0x10 + curMode); /* 0001MMMM */
      }
      break;
    case 0x04: /* Left pad 4 - fading strobe mode */
      if(value == 0x7F) {
        if(curMode == 2) {
          curMode = 0;
          midiPort.sendControllerChange(0, 0x04, 0);
        }
        else {
          curMode = 2;
          midiPort.sendControllerChange(0, 0x01, 0);
          midiPort.sendControllerChange(0, 0x04, 0x7F);
        }
        
        if(autoHueReset == 1)
          serialPort.write(0x20); /* 00100000 */
        
        serialPort.write(0x10 + curMode); /* 0001MMMM */
      }
      break;
    case 0x02: /* Left pad 2 - Reset hue */
      if(value == 0x7F) {
        serialPort.write(0x20); /* 00100000 */
        midiPort.sendControllerChange(0, 0x02, 0x7F);
      }
      else {
        midiPort.sendControllerChange(0, 0x02, 0);
      }
      break;
    default:
      break;
  }
}
